/*
 *
 *
 *
 *
 *
 */

var processCommand = "tasklist /FO CSV /V /NH";
var processList = new Array();
var processName = new Array();
var processNameShadow = new Array();
var processId = new Array();
var processIdShadow = new Array();
var processMem = new Array();
var processMemShadow = new Array();
var processCpu = new Array();
var processCpuShadow = new Array();
var backgroundImages = new Array();
var backgroundColumn = new Array();
 
function Process() {
  name = null;
  pid  = null;
  session = null;
  sessionNum = -1;
  memory = null;
  status = null;
  user = null;
  cpu = null;
  window = null;
}
  
/**
 *  executes the "processCommand" in the shell to get a string listing
 *  of operating system processes
 *
 *  returns a string of processes
 */
function getProcessString() {
  str = null;
  str = runCommand(processCommand);
  return str;
}
      
/**
 * populates the global variable "processList" with process objects 
 *
 * returns nothing
 */
function getProcessList() {
  str = getProcessString();
  if(str == null) {
    return null;
  }
  lines = str.split("\n");
  valid = 0;
  for(i=0; i < lines.length; i++) {
    if(lines[i].length < 2) continue;
    valid++;
  }
  processList = new Array(valid);
  for(i=0,j=0; i < lines.length; i++) {
    if(lines[i].length < 2) continue;
    lines[i] = removeQuotedComma(lines[i]);
    while(lines[i].indexOf("\"") > -1) {
      lines[i] = lines[i].replace("\"","");
    }
    fields = lines[i].split(",");
    processList[j] = new Process();
    processList[j].name = fields[0];
    processList[j].pid  = fields[1];
    processList[j].session = fields[2];
    processList[j].sessionNum = fields[3];
    processList[j].memory = fields[4];
    processList[j].status = fields[5];
    processList[j].user = fields[6];
    processList[j].cpu = fields[7];
    processList[j].window = fields[8];
    j++;
  }
  sortProcess();
}
    
function getSystemProcessCount() {
  cnt = 0;
  for(i=0; i < processList.length; i++) {
    if(processList[i].user.indexOf("NT AUTHORITY") > -1) {
      cnt++ 
    }
  }
  return cnt;
}    
    
/**
 * recursive function to remove commas inside double quotes
 */
function removeQuotedComma(str) {
  if((index = str.indexOf(",")) > -1) {
    good = "";
    if(str.charAt(index-1) == '"') {
      good = str.substring(0,index+1);
    } else {
      good = str.substring(0,index);
    }
    return good + removeQuotedComma(str.substring(index+1));
  } else {
    return str;
  }
}      
      
/**
 * converts memory usage strings to an integer
 * ex. 16,200 K = 16588800
 */
function convertMem2Number(str) {
  while(str.indexOf(",") > -1) {
    str = str.replace(",","");
  }
  var num = parseInt(str);
  if(str.charAt(str.length - 1) == 'K') {
    num = num * 1024;
  }
  return num;
}      

/**
 * converts cpu time strings to an integer
 */
function convertCpu2Number(str) {
  hs = 0;
  if(str.length == 7) {
    he = 1; ms = 2; ss = 5;
  } else if(str.length == 8) {
    he = 2; ms = 3; ss = 6;
  } else if(str.length == 9) {
    he = 3; ms = 4; ss = 7;
  }
  var num = parseInt(str.substr(hs,he)) * 3600;
  num += parseInt(str.substr(ms,2)) * 60;
  num += parseInt(str.substr(ss));
  return num;
}      
   
/////////////////////////////////////////////////////////////////////
// sorting functions
/////////////////////////////////////////////////////////////////////   
function sortProcess() {
  if(preferences.sortBy.value == "Process ID Low->High") {
    processList.sort(comparePidUp);
    return;
  }
  if(preferences.sortBy.value == "Process ID High->Low") {
    processList.sort(comparePidDown);
    return;
  }
  if(preferences.sortBy.value == "Memory Usage") {
    processList.sort(compareMemory);
    return;
  }
  if(preferences.sortBy.value == "Name") {
    processList.sort(compareName);
    return;
  }
  if(preferences.sortBy.value == "CPU Time") {
    processList.sort(compareCpu);
    return;
  }
}
      
function compareName(a,b) {
  if(a.name > b.name) return 1;
  if(a.name < b.name) return -1;
  return 0;
}      
function compareMemory(a,b) {
  return convertMem2Number(b.memory) - convertMem2Number(a.memory);
}   
function compareCpu(a,b) {
  return convertCpu2Number(b.cpu) - convertCpu2Number(a.cpu);
}   
function comparePidUp(a,b) {
  return parseInt(a.pid) - parseInt(b.pid);
}   
function comparePidDown(a,b) {
  return parseInt(b.pid) - parseInt(a.pid);
}   
   
/////////////////////////////////////////////////////////////////////
// display functions
/////////////////////////////////////////////////////////////////////      
var vertical = 36;
var horizontal = 40;
var backHeight = 20;
var bottomHome = 36;

function updateDisplay(doublearray) {
  suppressUpdates();
  system = getSystemProcessCount();
  totals.data = "Total: "+processList.length+"   System: "+system+"   User: "+Number(processList.length - system);
  
  if(preferences.opened.value == 0) {
    drawProcess(0);
    if(bottomImage.vOffset != bottomHome) {
      button.opacity = 0;
      bottomImage.moveTo(0,bottomHome,5);
      //button.hOffset = 172;
      button.vOffset = bottomHome+12;
      button.src = "images/arrow-down.png";
      button.fade(0,255,5);
    }
  } else {
    if(bottomImage.vOffset != preferences.listMax.value*backHeight+bottomHome) {
      button.opacity = 0;
      bottomImage.moveTo(0,preferences.listMax.value*backHeight+bottomHome,5);
      button.vOffset = preferences.listMax.value*backHeight+bottomHome+12;
      button.src = "images/arrow-up.png";
      button.fade(0,255,5);      
    } 
    drawProcess(preferences.listMax.value);
  }

  resumeUpdates();
}
      
      
function drawProcess(number) {
  while(processId.length > number) {
    item = backgroundImages.pop(); item.opacity = 0; delete item;
    //item = backgroundColumn.pop(); item.opacity = 0; delete item;    
    item = processId.pop(); item.opacity = 0; delete item;
    item = processIdShadow.pop(); item.opacity = 0; delete item;
    item = processName.pop(); item.opacity = 0; delete item;
    item = processNameShadow.pop(); item.opacity = 0; delete item;
    item = processMem.pop(); item.opacity = 0; delete item;
    item = processMemShadow.pop(); item.opacity = 0; delete item;
    item = processCpu.pop(); item.opacity = 0; delete item;
    item = processCpuShadow.pop(); item.opacity = 0; delete item;
  }

  while(backgroundImages.length < number) {
    index = backgroundImages.length;
    backgroundImages[index] = new Image();
    backgroundImages[index].src = "images/middle.png";
    backgroundImages[index].vOffset = vertical+(index*backHeight);
    backgroundImages[index].hOffset = 5;
    backgroundImages[index].opacity = 180;
    //backgroundImages[index].fade(0,200,2);
    //backgroundColumn[index] = new Image();
    //backgroundColumn[index].src = "images/42.png";
    //backgroundColumn[index].vOffset = vertical+(index*backHeight);
    //backgroundColumn[index].hOffset = 0;    
    //backgroundColumn[index].opacity = 80;    
  }
  
  for(i = 0; i < number; i++) {
    shadowedText("processId",i,processList[i].pid,12,"#FFFFFF",horizontal,vertical+(i*backHeight)+14,"right");
    shadowedText("processName",i,processList[i].name,12,"#FFFFFF",horizontal+10,vertical+(i*backHeight)+14,"left");
    shadowedText("processMem",i,processList[i].memory,12,"#FFFFFF",horizontal+240,vertical+(i*backHeight)+14,"right");
    shadowedText("processCpu",i,processList[i].cpu,12,"#FFFFFF",horizontal+300,vertical+(i*backHeight)+14,"right");
  }
}      
      
      
      
function open() {
  if(preferences.opened.value == 0) {
    preferences.opened.value = 1;
  } else {
    preferences.opened.value = 0;
  }
  updateDisplay();
}
      

function shadowedText(name, subname, content, size, color, hLoc, vLoc, alignDir)
{
	shadowName = name + "Shadow";

	eval(shadowName + "[" + subname + "]" + " = new Text()");
	eval(shadowName + "[" + subname + "]" + ".font = \"Helvetica Bold, Arial\"");
	eval(shadowName + "[" + subname + "]" + ".style = \"bold\"");
	eval(shadowName + "[" + subname + "]" + ".size = " + size);
	eval(shadowName + "[" + subname + "]" + ".color = \"#000000\"");
	eval(shadowName + "[" + subname + "]" + ".opacity = \"185\"");
	eval(shadowName + "[" + subname + "]" + ".data = \"" + content + "\"");
	eval(shadowName + "[" + subname + "]" + ".hOffset = " + hLoc);
	eval(shadowName + "[" + subname + "]" + ".vOffset = " + Number(vLoc - 1));
	eval(shadowName + "[" + subname + "]" + ".alignment = \"" + alignDir + "\"");

	eval(name + "[" + subname + "]" + " = new Text()");
	eval(name + "[" + subname + "]" + ".font = \"Helvetica Bold, Arial\"");
	eval(name + "[" + subname + "]" + ".style = \"bold\"");
	eval(name + "[" + subname + "]" + ".size = " + size);
	eval(name + "[" + subname + "]" + ".color = \"" + color + "\"");
	eval(name + "[" + subname + "]" + ".data = \"" + content + "\"");
	eval(name + "[" + subname + "]" + ".hOffset = " + hLoc);
	eval(name + "[" + subname + "]" + ".vOffset = " + vLoc);
	eval(name + "[" + subname + "]" + ".alignment = \"" + alignDir + "\"");
}


getProcessList();
updateDisplay();
